--[[
BY
 _____   _____   __   _  
|  _  \ /  _  \ |  \ | | 
| | | | | | | | |   \| | 
| | | | | | | | | |\   | 
| |_| | | |_| | | | \  | 
|_____/ \_____/ |_|  \_| 

]]

require "prefabutil"

local assets =
{	
	
	Asset("IMAGE", "images/minimap/dontreasurechestmap.tex"),
	Asset("ATLAS", "images/minimap/dontreasurechestmap.xml"),
	
	Asset("ANIM", "anim/ui_chest_3x2.zip"),
	
	Asset("ANIM", "anim/dontreasure_chest.zip"),
	Asset("ANIM", "anim/luggage.zip"),
	Asset("ANIM", "anim/octopus_chest.zip"),
	Asset("ANIM", "anim/kraken_chest.zip"),	
	Asset("ANIM", "anim/skull_chest.zip"),	
}
	
local prefabs =
{	
	"collapse_small",	
}

local chestmaxlife = 20
	
local function onopen(inst)
    if not inst:HasTag("burnt") then
        inst.AnimState:PlayAnimation("open")
		inst.AnimState:PlayAnimation("open")
		if inst.skinchesttag == "aquatic" then
			inst.AnimState:PushAnimation("opened", true)
		end
        inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
    end
end 

local function onclose(inst)
    if not inst:HasTag("burnt") then
        inst.AnimState:PlayAnimation("close")
		if inst.skinchesttag == "aquatic" then
			inst.AnimState:PushAnimation("closed", true)
		end		
        inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
    end
end

local function onhammered(inst, worker)
    if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
        inst.components.burnable:Extinguish()
    end
    inst.components.lootdropper:DropLoot()
    if inst.components.container ~= nil then
        inst.components.container:DropEverything()
    end
	
	if inst.components.container ~= nil then
		inst.components.container:DropEverything()
		inst.components.container:Close()
	end
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx:SetMaterial("wood")
    inst:Remove()
end

local function checkstatusdon(inst)
	local chestowner = inst.components.securitycontainerdon:GetOwnerSecurity()
	local strinfochest_owner, strinfochest_itens, strinfochest_life
	
	if chestowner ~= nil then
		
		strinfochest_owner = (STRINGS.DONTEXT_OWNER..": "..tostring(chestowner))
		strinfochest_itens = (STRINGS.DONTEXT_ITENS..": "..tostring(inst.components.container:NumItems()).."/"..tostring(inst.components.container:GetNumSlots()))
		
		if inst.indestructible == false and inst.components.workable ~= nil then
			--strinfochest_life = (STRINGS.DONTEXT_LIFE..": "..tostring(inst.components.workable.workleft).."/"..tostring(inst.components.workable.maxwork))
			strinfochest_life = (STRINGS.DONTEXT_LIFE..": "..tostring(inst.components.workable.workleft).."/"..tostring(chestmaxlife))
		else
			strinfochest_life = (STRINGS.DONTEXT_LIFE..": "..STRINGS.DONTEXT_INDESTRUCTIBLE)
		end	

		inst.components.talker:Say((strinfochest_owner.."\n"..strinfochest_itens.."\n"..strinfochest_life), 5)	
	else	
		inst.components.talker:Say(STRINGS.DONTEXT_WAITPW.."!", 5)	
	end		
end


local function checkupdateskinchest(inst)
	local idskin = inst.components.securitycontainerdon.currentidskinchest
	
	if idskin == 1 then
		inst.AnimState:SetBank("donchest")
        inst.AnimState:SetBuild("dontreasure_chest")
		inst.skinchesttag = ""
	elseif idskin == 2 then
		inst.AnimState:SetBank("sacred_chest")
        inst.AnimState:SetBuild("sacred_chest")	
		inst.skinchesttag = ""	
	elseif idskin == 3 then
		inst.AnimState:SetBank("pandoras_chest")
        inst.AnimState:SetBuild("pandoras_chest")
		inst.skinchesttag = ""
	elseif idskin == 4 then
		inst.AnimState:SetBank("dragonfly_chest")
        inst.AnimState:SetBuild("dragonfly_chest")
		inst.skinchesttag = ""	
	elseif idskin == 5 then
		inst.AnimState:SetBank("skull_chest")
        inst.AnimState:SetBuild("skull_chest")
		inst.skinchesttag = ""
	elseif idskin == 6 then
		inst.AnimState:SetBank("luggage")
        inst.AnimState:SetBuild("luggage")
		inst.skinchesttag = "aquatic"
	elseif idskin == 7 then
		inst.AnimState:SetBank("octopus_chest")
        inst.AnimState:SetBuild("octopus_chest")
		inst.skinchesttag = "aquatic"			
	elseif idskin == 8 then
		inst.AnimState:SetBank("kraken_chest")
        inst.AnimState:SetBuild("kraken_chest")
		inst.skinchesttag = "aquatic"		
	end	
	
	
	if inst.skinchesttag == "aquatic" then
		inst.AnimState:PushAnimation("closed", true)
	end	
	
end

local function updateskinschest(inst)
	checkupdateskinchest(inst)
	checkstatusdon(inst)
end

local function destroyoncancel(inst)
	if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
        inst.components.burnable:Extinguish()
    end
    if inst.components.container ~= nil then
        inst.components.container:DropEverything()
    end
	
	SpawnPrefab("boards").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("boards").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("boards").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("boards").Transform:SetPosition(inst.Transform:GetWorldPosition())
	
	SpawnPrefab("goldnugget").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("goldnugget").Transform:SetPosition(inst.Transform:GetWorldPosition())
	
	
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx:SetMaterial("wood")
    inst:Remove()
end

local function onhit(inst, worker)
    if not inst:HasTag("burnt") then
        inst.AnimState:PlayAnimation("hit")
        inst.AnimState:PushAnimation("closed", true) 
    end
		checkstatusdon(inst)
end

local function checkindestructible(inst)
	if inst.indestructible == false then
		inst:AddComponent("lootdropper")
		inst:AddComponent("workable")
		inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
		inst.components.workable:SetWorkLeft(chestmaxlife)
		inst.components.workable:SetOnFinishCallback(onhammered)
		inst.components.workable:SetOnWorkCallback(onhit)

		--MakeSmallBurnable(inst, nil, nil, true)
		--MakeMediumPropagator(inst)
	end
end

local function onbuilt(inst, builder)
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("closed", true)
    inst.SoundEmitter:PlaySound("dontstarve/common/chest_craft")
	checkindestructible(inst)
	updateskinschest(inst)
end

local function onsave(inst, data)
    if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() or inst:HasTag("burnt") then
        data.burnt = true
    end
end

local function onload(inst, data)
    if data ~= nil and data.burnt and inst.components.burnable ~= nil then
        inst.components.burnable.onburnt(inst)
    end
	checkindestructible(inst)
	updateskinschest(inst)		
end


local function MakeChestSecuritydon(name, bank, build)
    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()
        inst.entity:AddMiniMapEntity()
        inst.entity:AddNetwork()

        inst.MiniMapEntity:SetIcon("dontreasurechestmap.tex")

        inst:AddTag("structure")
        inst:AddTag("chest")
		inst:AddTag("_securitycontainerdon")	
		
		inst.Transform:SetScale(1.25, 1.25, 1.25)

        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:PlayAnimation("closed", true)

        MakeSnowCoveredPristine(inst)
	
		inst:AddComponent("talker")
		inst.components.talker.fontsize = 22
		inst.components.talker.font = UIFONT
		inst.components.talker.offset = Vector3(0,-325,0)				

        inst.entity:SetPristine()

        if not TheWorld.ismastersim then	
			inst.OnEntityReplicated = function(inst) inst.replica.container:WidgetSetup("shadowchester") end	
		return inst
		end


		inst:RemoveTag("_securitycontainerdon")

		inst:AddComponent("securitycontainerdon")
		inst.components.securitycontainerdon.ondestroyoncancelfn = destroyoncancel
		inst.components.securitycontainerdon.onupdateskinschestfn = updateskinschest
		inst.components.securitycontainerdon.maxcurrentidskinchest = 8

        inst:AddComponent("inspectable")
        inst:AddComponent("container")
        inst.components.container:WidgetSetup("shadowchester")
        inst.components.container.onopenfn = onopen
        inst.components.container.onclosefn = onclose
        inst.components.container.canbeopened = false
		
		inst:DoPeriodicTask(2, checkstatusdon)

        inst:AddComponent("hauntable")
        inst.components.hauntable:SetHauntValue(TUNING.HAUNT_TINY)

        inst:ListenForEvent("onbuilt", onbuilt)
        MakeSnowCovered(inst)

        inst.OnSave = onsave 
        inst.OnLoad = onload

        return inst
    end

    return Prefab(name, fn, assets, prefabs)
end


return MakeChestSecuritydon("dontreasurechest", "donchest", "dontreasure_chest"),
	   MakePlacer("dontreasurechest_placer", "donchest", "dontreasure_chest", "closed")
	   